#include <stdio.h>
#include <glib.h>
#include <string.h>
#include <stdlib.h>
#include "causalito.h"
#include "misc.h"
#include "model.h"
#include "globals.h"

/**
 * alloc_and_init_model:
 * @void: 
 * 
 * 
 * 
 * Return value: 
 **/
Model alloc_and_init_model(void)
{
  Model m;
  
  m = (Model) malloc(sizeof(struct _Model));
  m->eqs_name_list = NULL;
  m->vars_name_list = NULL;
  m->conds_name_list = NULL;
  m->eqs_list = NULL;
  m->confs_list = NULL;
  m->neqs = 0;
  m->nvars = 0;
  m->nconds = 0;
  m->nconfs = 0;
  return m;
}

/**
 * alloc_and_init_equation:
 * @void: 
 * 
 * 
 * 
 * Return value: 
 **/
Equation alloc_and_init_equation(void)
{
  Equation e;
  
  e = (Equation) malloc(sizeof(struct _Equation));
  e->comm = NULL;
  e->comp = NULL;
  e->eq = NULL;
  e->cond = NULL;
  e->varf = NULL;
  e->depc = NULL;
  e->matched = NULL;
  e->vars = NULL;
  e->nvars = 0;
  e->positive = True;
  return e;
}

/**
 * add_eq_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_eq_in_equation(Equation e, char * s)
{
  if (s && strlen(s))
    e->eq = strdup(s);
  return SUCCESS;
}

/**
 * add_var_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_var_in_equation(Equation e, char * s)
{
  if (s && strlen(s) && !is_string_in_list(s, e->vars))
    e->vars = g_slist_append(e->vars, strdup(s));
  return SUCCESS;
}

/**
 * add_varf_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_varf_in_equation(Equation e, char * s)
{
  if (s && strlen(s))
    e->varf = strdup(s);
  return SUCCESS;
}

/**
 * add_comm_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_comm_in_equation(Equation e, char * s)
{
  if (s && strlen(s))
    e->comm = strdup(s);
  return SUCCESS;
}

/**
 * add_comp_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_comp_in_equation(Equation e, char * s)
{
  if (s && strlen(s))
    e->comp = strdup(s);
  return SUCCESS;
}

/**
 * add_cond_in_equation:
 * @e: 
 * @s: 
 * 
 * 
 * 
 * Return value: 
 **/
int add_cond_in_equation(Equation e, char * s)
{
  if (s && strlen(s))
    e->cond = strdup(s);
  else e->cond = strdup("t");
  return SUCCESS;
}

Equation is_eq_in_model(char * str)
{
  int i;
  Equation e;
  
  e = NULL;
  for (i = 0 ; i < model->neqs ; i++) {
    e = g_slist_nth_data(model->eqs_list, i);
    if (!strcmp(e->eq, str))
      return e;
  }
  return e;
}

/* int del_var_in_equation(Equation, char *); */
/* int del_varf_in_equation(Equation, char *); */
/* int del_comm_in_equation(Equation, char *); */
/* int del_comp_in_equation(Equation, char *); */
/* int del_cond_in_equation(Equation, char *); */

/* void free_model(void); */
/* void free_equation(void); */
/* void free_options(void); */

/*** Local Variables: ***/
/*** change-log-default-name: "../ChangeLog" ***/
/*** End: ***/
