#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "graph.h"
#define SUCCESS 1
#define FAILURE 0
#define is_string_in_list(s, l) (g_slist_find_custom((l), (s), (GCompareFunc) strcmp) ? SUCCESS : FAILURE)
Graph G;

int main(void)
{
  GSList * successors, * VprimeC;
  
  G = (Graph) malloc(sizeof(struct _Graph));
  G->nodes_list = NULL;
  G->arcs_list = NULL;

  add_new_arc_to_graph("A", "B", G);
  add_new_arc_to_graph("A", "C", G);
  add_new_arc_to_graph("B", "D", G);
  add_new_arc_to_graph("B", "F", G);
  add_new_arc_to_graph("C", "E", G);
  add_new_arc_to_graph("D", "H", G);
  add_new_arc_to_graph("E", "G", G);
  add_new_arc_to_graph("H", "C", G);

  VprimeC = NULL;
  VprimeC = g_slist_append(VprimeC, "H");
  VprimeC = g_slist_append(VprimeC, "B");

  successors = NULL;
  successors = search_successors(successors, G, VprimeC);

  while (successors != NULL) {
    printf("%s ", successors->data);
    successors = successors->next;
  }
  printf("\n");
    
  return 1;
}
/**
 * is_node_in_graph: Search a node by its name in a graph.
 * @s: the name of the node to be searched for.
 * @g:  the graphe in which the node is searched.
 * 
 * 
 * 
 * Return value: a pointer on the searched node or NULL if the node
 * does not exists
 **/
Node is_node_in_graph(char * s, Graph g)
{
  int l, i;
  Node n;

  l = g_slist_length(g->nodes_list);
  for (i = 0 ; i < l ; i++) {
    n = g_slist_nth_data(g->nodes_list, i);
    if (!strcmp(n->name, s)) {
      return n;
    }
  }
  return NULL;
}

/**
 * is_arc_in_graph: Search for the arc given the initial and final
 * nodes names in a graph
 * @s: the name of the initial node of the arc.
 * @e: the name of the ending node of the arc.
 * @g: 
 * 
 * 
 * 
 * Return value: a pointer on the found arc or NULL if the arc does
 * not exists.
 **/
Arc is_arc_in_graph(char * s, char * e, Graph g)
{
  int l, i;
  Arc a;
  
  l = g_slist_length(g->arcs_list);
  for (i = 0 ; i < l ; i++) {
    a = g_slist_nth_data(g->arcs_list, i);
    if (!strcmp((a->start)->name, s) && !(strcmp((a->end)->name, e))) {
      return a;
    }
  }
  return NULL;
}


/**
 * add_new_node_to_graph:
 * @s: 
 * @g: 
 * 
 * 
 * 
 * Return value: 
 **/
Node add_new_node_to_graph(char * s, Graph g)
{
  Node n;

  n = (Node) malloc(sizeof(struct _Node));
  n->name = strdup(s);
  n->pred = NULL;
  n->delta = 0;
  g->nodes_list = g_slist_append(g->nodes_list, n);
  return n;
}

/**
 * add_new_arc_to_graph:
 * @s: 
 * @e: 
 * @g: 
 * 
 * 
 * 
 * Return value: 
 **/
Arc add_new_arc_to_graph(char * s, char * e, Graph g)
{
  Node ns, ne;
  Arc a;
  
  if (!(ns = is_node_in_graph(s, g)))
    ns = add_new_node_to_graph(s, g);
  if (!(ne = is_node_in_graph(e, g)))
    ne = add_new_node_to_graph(e, g);
  a = (Arc) malloc(sizeof(struct _Arc));
  a->start = ns;
  a->end = ne;
  *(a->label) = '\0'; 
  g->arcs_list = g_slist_append(g->arcs_list, a);
  return a;
}

/**
 * debug_print_graph:
 * @g: 
 * 
 * 
 * 
 * Return value: 
 **/
int debug_print_graph(Graph g)
{
  int i, l;

  l = g_slist_length(g->arcs_list);
  for (i = 0 ; i < l ; i++)
    debug_print_arc(g_slist_nth_data(g->arcs_list, i));
  l = g_slist_length(g->nodes_list);
  for (i = 0 ; i < l ; i++)
    debug_print_node(g_slist_nth_data(g->nodes_list, i));
  return 0;
}

/**
 * debug_print_arc:
 * @a: 
 * 
 * 
 * 
 * Return value: 
 **/
int debug_print_arc(Arc a)
{
  printf("Arc: %5s %5s %5d %5d %s\n", (a->start)->name, (a->end)->name,
	   a->flow, a->capacity, a->label);
  return 0;
}

/**
 * debug_print_node:
 * @n: 
 * 
 * 
 * 
 * Return value: 
 **/
int debug_print_node(Node n)
{
  printf("Noeud: %5s %5s %5d\n", n->name,
	 n->pred != NULL ? n->pred->name : "NULL", n->delta);
  return 0;
}

GSList * search_successors(GSList * GammaVprimeC, Graph GprimeC0,
			   GSList * VprimeC)
{
  short unsigned int C[1024][1024];
  GSList * varlist;
  int n, i, j, k, nv;
  char * d, * f;
  Node noeud;
  
  /* collect the list of variables in GprimeC0 */
  n = g_slist_length(GprimeC0->nodes_list);
  for (i = 0 ; i < n ; i++) {
    noeud = g_slist_nth_data(GprimeC0->nodes_list, i);
    varlist = g_slist_append(varlist, noeud->name);
  }
  /* building adjacency matrix of the GprimeC0 graph */
  n = g_slist_length(varlist);
  for (i = 0 ; i < n ; i++) {
    d = g_slist_nth_data(varlist, i);
    for (j = 0 ; j < n ; j++) {
      f = g_slist_nth_data(varlist, j);
      if (is_arc_in_graph(d, f, GprimeC0))
	C[i][j] = 1;
      else C[i][j] = 0;
    }
  }
  /* compute transitive closure of graph (Warshall algorithm) */
  for (i = 0 ; i < n ; i++)
    for (j = 0 ; j < n ; j++)
      for (k = 0 ; k < n ; k++)
	C[i][j] = C[i][j] || (C[i][k] && C[k][j]);
  /* search for successors */
  for (i = 0 ; i < n ; i++) {
    d = g_slist_nth_data(varlist, i);
    if (is_string_in_list(d, VprimeC)) {
      for (j = 0 ; j < n ; j++) {
	if (C[i][j]) {
	f = g_slist_nth_data(varlist, j);
	if (!is_string_in_list(f, GammaVprimeC))
	  GammaVprimeC = g_slist_append(GammaVprimeC, f);
	}
      }
    }
  }
  return GammaVprimeC;
}

  

/*** Local Variables: ***/
/*** change-log-default-name: "../ChangeLog" ***/
/*** End: ***/
